/// <reference path="../../reference/cristalix.d.ts" />
import { AnimationFinisher, ElementData, RectData, ItemData, TextData, Rotation, V3 } from './api';
export declare const buffer: FloatBuffer;
export declare abstract class AbstractElement {
    properties: number[];
    matrices: Matrix4f[];
    enabled: boolean;
    dirtyMatrices: number[];
    hovered: boolean;
    beforeRender?: () => void;
    afterRender?: () => void;
    passedHoverCulling: boolean;
    cachedHexColor: number;
    onHover?: (element: AbstractElement, hovered: boolean) => void;
    onClick?: (element: AbstractElement, down: boolean, button: number) => void;
    constructor(data: ElementData);
    applyTransformations(): void;
    setOffset(offset: V3, duration?: number, easingId?: number, onFinish?: AnimationFinisher): void;
    setAlign(align: V3, duration?: number, easingId?: number, onFinish?: AnimationFinisher): void;
    setOrigin(origin: V3, duration?: number, easingId?: number, onFinish?: AnimationFinisher): void;
    setScale(scale: V3, duration?: number, easingId?: number, onFinish?: AnimationFinisher): void;
    setRotation(rotation: Rotation, duration?: number, easingId?: number, onFinish?: AnimationFinisher): void;
    setProperty(propertyId: number, value: number, duration?: number, easingId?: number, onFinish?: AnimationFinisher): void;
    markDirty(matrixIds: number[]): void;
    cleanMatrices(): void;
    updateMatrix(matrixId: number): void;
    transformAndRender(): void;
    protected abstract render(): void;
}
export declare class TextElement extends AbstractElement {
    text?: string;
    autoFit: boolean;
    shadow: boolean;
    constructor(data: TextData);
    updateMatrix(matrixId: number): void;
    setText(text: string): void;
    render(): void;
}
export declare class RectangleElement extends AbstractElement {
    children: AbstractElement[];
    texture: string | ResourceLocation;
    constructor(data: RectData);
    updateMatrix(matrixId: number): void;
    render(): void;
}
export declare class ItemElement extends AbstractElement {
    item: ItemStack;
    constructor(data: ItemData);
    render(): void;
}
